@@ -36,7 +36,7 @@ test_create_cursor_empty_query (EbSqlFixture *fixture,
 		fixture->ebsql, NULL,
 		sort_fields, sort_types, 2, &error);
 
-	g_assert (cursor != NULL);
+	g_assert_true (cursor != NULL);
 	e_book_sqlite_cursor_free (fixture->ebsql, cursor);
 }
 
@@ -58,7 +58,7 @@ test_create_cursor_valid_query (EbSqlFixture *fixture,
 		fixture->ebsql, sexp,
 		sort_fields, sort_types, 2, &error);
 
-	g_assert (cursor != NULL);
+	g_assert_true (cursor != NULL);
 	e_book_sqlite_cursor_free (fixture->ebsql, cursor);
 	g_free (sexp);
 	e_book_query_unref (query);
@@ -77,7 +77,7 @@ test_create_cursor_invalid_sort (EbSqlFixture *fixture,
 		fixture->ebsql, NULL,
 		sort_fields, sort_types, 1, &error);
 
-	g_assert (cursor == NULL);
+	g_assert_true (cursor == NULL);
 	g_assert_error (error, E_BOOK_SQLITE_ERROR, E_BOOK_SQLITE_ERROR_INVALID_QUERY);
 	g_clear_error (&error);
 }
@@ -91,7 +91,7 @@ test_create_cursor_missing_sort (EbSqlFixture *fixture,
 
 	cursor = e_book_sqlite_cursor_new (fixture->ebsql, NULL, NULL, NULL, 0, &error);
 
-	g_assert (cursor == NULL);
+	g_assert_true (cursor == NULL);
 	g_assert_error (error, E_BOOK_SQLITE_ERROR, E_BOOK_SQLITE_ERROR_INVALID_QUERY);
 	g_clear_error (&error);
 }
@@ -104,9 +104,12 @@ main (gint argc,
 	g_type_init ();
 #endif
 	g_test_init (&argc, &argv, NULL);
+	g_test_bug_base ("https://gitlab.gnome.org/GNOME/evolution-data-server/");
+
+	data_test_utils_read_args (argc, argv);
 
 	/* Ensure that the client and server get the same locale */
-	g_assert (g_setenv ("LC_ALL", "en_US.UTF-8", TRUE));
+	g_assert_true (g_setenv ("LC_ALL", "en_US.UTF-8", TRUE));
 	setlocale (LC_ALL, "");
 
 	g_test_add (
@@ -122,5 +125,5 @@ main (gint argc,
 		"/EbSqlCursor/Create/MissingSort", EbSqlFixture, &closure,
 		e_sqlite_fixture_setup, test_create_cursor_missing_sort, e_sqlite_fixture_teardown);
 
-	return e_test_server_utils_run_full (0);
+	return e_test_server_utils_run_full (argc, argv, 0);
 }
